package industries.sunshine.planningpoker

import scala.scalajs.js
import com.raquo.laminar.api.L.{*, given}
import industries.sunshine.planningpoker.common.Models.*
import io.laminext.fetch.Fetch
import scala.scalajs.js.Dynamic.{global => g}

import concurrent.ExecutionContext.Implicits.global

object OwnHandControls {
  def render(roomStateSignal: Signal[RoomStateView]): Element = {
    val cardTypesSignal = roomStateSignal.map(myUnselectedCards(_))
    div(
      className := " h-1/3 relative",
      children <-- cardTypesSignal.map(cards => cards.map(renderCard(_, cards.size))),
    )
  }

  def dynamicCardStyle(index: Int, totalCards: Int): String = {
    val offCenterIndex = 2*index - (totalCards + 1)
    val maxOffCenter = (totalCards + 1) / 2
    val angle = Math.round((offCenterIndex / maxOffCenter.toDouble) * 15)
    val xOffset = offCenterIndex * 1.3
    s"$offCenterIndex ${(offCenterIndex.toLong / maxOffCenter.toLong)} transform: rotate(${angle}deg) "
    s"transform: rotate(${angle}deg) translateX(${xOffset}em) translateY(-2em)"
  }

  private def renderCard(cardWithIndex: (String, Int), totalCards: Int): Element = {
    val (value, index) = cardWithIndex
    val submitVote = Fetch.get(s"/api/vote/$value").text

    div(
      className := "cursor-pointer w-24 h-48 m-1 rounded-l flex justify-center items-center m-3 text-black bg-gray-50 border-black border-2 drop-shadow-md",
      className := " hover:z-50 hover:drop-shadow-xl rounded-lg hover:bg-gray-100 ease-linear duration-200",
      className := "absolute origin-bottom start-1/2",
      styleAttr := dynamicCardStyle(index, totalCards),
      onClick.flatMap(_ => submitVote) --> Observer(resp => g.console.info(resp.toString())),
      div(
        className := "-rotate-45 text-xl",
        value,
      )
    )
  }

  private def myUnselectedCards(state: RoomStateView): List[(String, Int)] = {
    val allCards = state.allowedCards
    val cards = state.round match {
      case RoundStateView.Voting(myCard, _) =>
        allCards.filterNot(value => myCard.contains(value))
      case RoundStateView.Viewing(votes) =>
        allCards.filterNot(value => votes.toMap.get(state.me).contains(value))
    }

    cards.zip(1 to state.allowedCards.size)
  }

}
